#include "outdoor_module.h"

/* ------------------------------------------------------------------- MACROS */



/* ---------------------------------------------------------------- VARIABLES */

static module_t         outdoor_module;

#if (_MMB_MODULE_WIFI == 1)

    /**
     * Wifi specific variables
     */
    static int32_t retVal;
    static uint16_t i;

#endif

/**
 * General variables
 */
static char text[30];

/* -------------------------------------------- PRIVATE FUNCTION DECLARATIONS */



/* ----------------------------------------------------------- IMPLEMENTATION */

void outdoor_module_init ( module_t ** module )
{
    /**
    * Set outdoor module state to inactive
    */
    outdoor_module.state = MODULE_INACTIVE;

    #if (_MMB_MODULE_WIFI == 1)

        /**
         * Initialize wifi device
         */
        bsp_wifi_init(  );
        
        /**
         * Set SPI1 to active state
         * !!! Skipping to set SPI1 module to active state will disable Touch panel
         */
        SPI_Set_Active( SPI1_Read, SPI1_Write );

        /**
         * Configure wifi device
         */
        
        /**
         * Initialize wifi app variables
         */
        retVal = initializeAppVariables( );
        
        /**
         * No need to call this function
         * This function logs error messages based on retVal from previously called function
         */
        api_wifi_prerror( retVal );

        /**
         * Configure device in default state
         */
        retVal = configureSimpleLinkToDefaultState( );

        if ( retVal < 0 )
        {
            if ( DEVICE_NOT_IN_STATION_MODE == retVal )
            {
                /**
                 * Failed to configure the device
                 */
            }

            /**
             * Wifi module will not be initialized as function will end before that
             */
            return;
        }

        /**
         * Device is configured in default state
         */

        retVal = sl_Start( 0, 0, 0 );

        if ( ( retVal < 0 ) || ( ROLE_STA != retVal ) )
        {
            /**
             * Failed to start the device
             * Wifi module will not be initialized as function will end before that
             */
            return;
        }
        
        /**
         * Device started as STATION
         */

    #endif

    /**
    * Initialize outdoor module
    */
    *module = &outdoor_module;
}

void outdoor_module_task ( void )
{
    switch ( outdoor_module.state )
    {
        case MODULE_INACTIVE :
        {
            /**
             * Draw screen elements
             */
            DrawImage( &Image_00_01_OUTDOOR );
            
            /**
             * Set outdoor module state to active
             */
            outdoor_module.state = MODULE_ACTIVE;

            /**
             * Approx. 1 min refresh rate
             */
            i = 65535;

            break;
        }
        case MODULE_ACTIVE :
        {
            #if ( _MMB_MODULE_WIFI == 1 )

                if (i == 65535)
                {
                    i = 0;
                
                    /**
                     * Establishing connection with AP
                     */
                    retVal = establishConnectionWithAP( );

                    if ( retVal < 0 )
                    {
                        /**
                         * Failed to establish connection w/ an AP
                         */
                        return;
                    }

                    /**
                     * Connection established w/ AP and IP is acquired
                     */
                     
                    /**
                     * Get weather data
                     */
                    retVal = getWeather( );

                    if ( retVal < 0 )
                    {
                        /**
                         * Failed to get weather information
                         */
                        return;
                    }

                    /**
                     * Disconnect from AP
                     */
                    retVal = disconnectFromAP( );

                    if ( retVal < 0 )
                    {
                        /**
                         * Failed to disconnect from AP
                         */
                        return;
                    }
                    
                    /**
                     * Clear all weather labels
                     */
                    Label_00_02_Outdoor_Temperature.Font_Color = CL_WHITE;
                    Label_00_03_Outdoor_Humidity.Font_Color = CL_WHITE;
                    Label_00_04_Outdoor_Pressure.Font_Color = CL_WHITE;
                    Label_00_05_Outdoor_Weather.Font_Color = CL_WHITE;
                    DrawLabel( &Label_00_02_Outdoor_Temperature );
                    DrawLabel( &Label_00_03_Outdoor_Humidity );
                    DrawLabel( &Label_00_04_Outdoor_Pressure );
                    DrawLabel( &Label_00_05_Outdoor_Weather );
                    
                    /**
                     * Get new weather data and set new weather label values
                     */
                    get_weather_station_data( Label_00_02_Outdoor_Temperature_Caption,
                                              Label_00_03_Outdoor_Humidity_Caption,
                                              Label_00_04_Outdoor_Pressure_Caption,
                                              Label_00_05_Outdoor_Weather_Caption );
                    
                    /**
                     * Draw weather labels
                     */
                    Label_00_02_Outdoor_Temperature.Font_Color = CL_BLACK;
                    Label_00_03_Outdoor_Humidity.Font_Color = CL_BLACK;
                    Label_00_04_Outdoor_Pressure.Font_Color = CL_BLACK;
                    Label_00_05_Outdoor_Weather.Font_Color = CL_BLACK;
                    DrawLabel( &Label_00_02_Outdoor_Temperature );
                    DrawLabel( &Label_00_03_Outdoor_Humidity );
                    DrawLabel( &Label_00_04_Outdoor_Pressure );
                    DrawLabel( &Label_00_05_Outdoor_Weather );
                }
                else
                {
                    i++;
                }
                
                /**
                 * Draw temperature and als labels
                 */
                Check_TP( );
                
            #endif

            break;
        }
        default :
        {
            /** 
             * Error 
             */
            while (1);
        }
    }
}

/* --------------------------------------------- PRIVATE FUNCTION DEFINITIONS */