/*
    interrupt.c

-----------------------------------------------------------------------------

  This file is part of mikroSDK.
  
  Copyright (c) 2017, MikroElektonika - http://www.mikroe.com

  All rights reserved.

----------------------------------------------------------------------------- */

/**
        \file   interrupt.c

- Version               :   1.0.0
- Date                  :   May 2019
- Developer             :   Milos Vidojevic

    Common interrupt module.

    Hints :

    - Copy file inside [project base directory]/portable.
    - Replace all "---" appearances with proper values.

*/
/* -------------------------------------------------------------------------- */

#include "simplelink.h"

#if ( _MMB_MODULE_WIFI == 1 )

/*
    WiFi interrupt related functions and variables.
*/

P_EVENT_HANDLER wifi_isr_event;

void bsp_wifi_driver_interrupt_enable ( void )
{
    //  Enable interrupt vector for CC3100 INT pin.

    NVIC_IntEnable( IVT_INT_EXTI0 );
}

void bsp_wifi_driver_interrupt_disable ( void )
{
    //  Disable interrupt vector for CC3100 INT pin.

    NVIC_IntDisable( IVT_INT_EXTI0 );
}

//  WiFi ISR Function.

void WIFI_EXT_ISR ( void ) iv IVT_INT_EXTI0 ics ICS_AUTO
{
    //  Reset interrupt flag

    EXTI_PR |= 1;
    
//    test_console_write_text( "|" );
    
    if ( wifi_isr_event != ( void * ) 0 )
    {
        ( *wifi_isr_event )( 0 );
    }
}

#endif

/*
    USB interrupt related functions and variables.
*/

void USB_ISR( ) iv IVT_INT_OTG_FS
{
    USBHost_IntHandler( );
}

/*
    Common initialize functions which should initialize all necessary
    interrupts.
*/

void interrupts_enable ( void )
{
    /*
        Initialize external interrupt on CC3100 INT pin.

        NOTE :

            Interrupt must be configured to trigger on rising edge.
    */

    SYSCFGEN_bit = 1;
    SYSCFG_EXTICR1 |= 0x00000006;
    EXTI_RTSR |= 1ul << 0;
    EXTI_IMR |= 1ul << 0;

    /*
        Initialize global interrupts and enable vectors.

            - Enable interrupt vector for USB.
            - Enable interrupt vector for CC3100 INT pin.
    */

    NVIC_IntEnable( IVT_INT_OTG_FS );
    NVIC_IntEnable( IVT_INT_EXTI0 );
    EnableInterrupts( );
}