/*
    mmb_---.h

-----------------------------------------------------------------------------

  This file is part of mikroSDK.
  
  Copyright (c) 2017, MikroElektonika - http://www.mikroe.com

  All rights reserved.

----------------------------------------------------------------------------- */

/**
        \file   mmb_---.h

- Version               :   1.0.0
- Date                  :   May 2019
- Developer             :   Milos Vidojevic

    Generated from mmb---.h template.

    ---

    Template for definition of MCU connections used on mikromedia systems. This 
    file should be used as start point for board definition file development. 

    Hints :

    - Copy file inside [project base directory]/portable.
    - Replace all "---" appearances with proper values.

*/
/* -------------------------------------------------------------------------- */

#include "compiler.h"

#ifndef _MMB_5_FPI_STM32_
#define _MMB_5_FPI_STM32_

/*
    REF SEL jumper position :

    3V3         - 1
    2.048V      - 0
*/

#define JP_VREF                     0

/*
    MP3 jumper position :

    LIN         - 1
    MIC         - 0
*/

#define JP_MP3                      0

/*
    ACCEL jumpers position.
*/

#define JP_ACCEL_A                  0
#define JP_ACCEL_B                  0

/*
    TFT jumpers position.
*/

#define JP_TFT_LR                   1
#define JP_TFT_UD                   0
#define JP_TFT_DITH                 1
#define JP_TFT_MODE                 0

// ============================================================================
//                                                                    DISPLAY
// ============================================================================

extern sfr sbit TFT_BLED;
extern sfr sbit TFT_CS;
extern sfr sbit TFT_RD;
extern sfr sbit TFT_RS;
extern sfr sbit TFT_RST;
extern sfr sbit TFT_WR;

static void Write_to_Data_Lines ( unsigned char _hi, unsigned char _lo )
{
    GPIOE_ODR = ( unsigned int ) ( _lo | ( _hi << 8 ) );
}

static void Set_Index ( unsigned short index )
{
    TFT_RS = 0;
    Write_to_Data_Lines( 0, index );
    TFT_WR = 0;
    asm nop;
    TFT_WR = 1;
}

static void Write_Command ( unsigned short cmd )
{
    TFT_RS = 1;
    Write_to_Data_Lines( 0, cmd );
    TFT_WR = 0;
    asm nop;
    TFT_WR = 1;
}

static void Write_Data ( unsigned int _data )
{
    TFT_RS = 1;
    Write_to_Data_Lines( ( _data >> 8 ), ( _data & 0x00FF ) );
    TFT_WR = 0;
    asm nop;
    TFT_WR = 1;
}

static void hal_display_init ( void )
{
    //  Initialize display.

    TFT_Set_Default_Mode( );
    TFT_Set_Active( Set_Index, Write_Command, Write_Data );
    TFT_Init_SSD1963_Board_70( 800, 480 );
    TFT_Set_DBC_SSD1963( 255 );
}

// ============================================================================
//                                                                      TOUCH
// ============================================================================

static uint8_t hal_touch_init ( void )
{
    //  Initialize touch senor.

    GPIO_Digital_Output( &GPIOB_BASE, _GPIO_PINMASK_6 );
    GPIO_Digital_Input( &GPIOB_BASE, _GPIO_PINMASK_7 );

    GPIOB_ODR.B6 = 1;

    while ( GPIOB_IDR.B7 == 0 )
    {
        GPIOB_ODR.B6 = 0;
        Delay_us( 10 );
        GPIOB_ODR.B6 = 1;
        Delay_us( 10 );
    }

    I2C1_Init_Advanced( 400000, &_GPIO_MODULE_I2C1_PB67 );

    FT5XX6_SetI2CAddress( 0x38 );

    if ( FT5XX6_IsOperational( ) != FT5XX6_OK )
    {
        return FT5XX6_FAILURE;
    }

    FT5XX6_SetDefaultMode( );
    FT5XX6_SetController( _TC_FT5X16 );
    FT5XX6_SetSizeAndRotation( 800, 480, 2 );

    return FT5XX6_OK;
}

// ============================================================================
//                                                                    CONSOLE
// ============================================================================

//  Map proper functions used with console.

#define CONSOLE_WRITE_TXT             UART6_Write_Text
#define CONSOLE_READY                 UART6_Data_Ready
#define CONSOLE_READ                  UART6_Read

static void hal_console_init ( void )
{
    //  Initialize console.

    UART6_Init_Advanced(
                         115200,
                         _UART_8_BIT_DATA,
                         _UART_NOPARITY,
                         _UART_ONE_STOPBIT,
                         &_GPIO_MODULE_USART6_PC67
                        );
}

// ============================================================================
//                                                                      POWER
// ============================================================================

#if ( _MMB_MODULE_POWER == 1 )

//  Detects external power source on TB1 / CN9.

static sbit BSP_DC_STATUS at                GPIOF_IDR.B6;

//  Detects charging of the battery.

static sbit BSP_CHARGER_STATUS at           GPIOF_IDR.B3;

#if ( JP_VREF == 0 )
#   define BSP_POWER_ADC_VREF               ( 2.048f )
#elif ( JP_VREF == 1 )
#   define BSP_POWER_ADC_VREF               ( 3.300f )
#else
#   error "REF SEL POSITION UNKNOWN"
#endif

#define BSP_POWER_ADC_CHANNEL               ( 15 )
#define BSP_POWER_ADC_RESOLUTION            ( 4096.0f )

const static hal_adc_obj_t BSP_POWER_ADC_MODULE =
{
    ADC3_Read,
    ADC3_Get_Sample
};

static void hal_power_init ( void )
{
    //  Initialize BSP_DC_STATUS as input pin.

    GPIO_Digital_Input( &GPIOF_BASE, _GPIO_PINMASK_6 );

    //  Initialize BSP_CHARGER_STATUS as input pin.

    GPIO_Digital_Input( &GPIOF_BASE, _GPIO_PINMASK_3 );

    //  Initialize ADC module.

    GPIO_Analog_Input( &GPIOF_BASE, _GPIO_PINMASK_5 );

    //  Initialize ADC module.

    ADC_Set_Input_Channel( BSP_POWER_ADC_CHANNEL );
    ADC3_Init( );
}

#endif

// ============================================================================
//                                                                    MICROSD
// ============================================================================

#if ( _MMB_MODULE_MICROSD == 1 )

//  Navigate to gpio.c file and update sbit variables Mmc_Chip_Select.

const static hal_spi_obj_t BSP_MICROSD_SPI_MODULE =
{
    SPI1_Write,
    SPI1_Read
};

static void hal_microsd_init ( void )
{
    //  Initialize Mmc_Chip_Select as output pin.

    GPIO_Digital_Output( &GPIOD_BASE, _GPIO_PINMASK_2);

    //  Initialize SPI module.

    SPI1_Init_Advanced( _SPI_FPCLK_DIV16,
                        _SPI_MASTER |
                        _SPI_8_BIT |
                        _SPI_CLK_IDLE_LOW |
                        _SPI_FIRST_CLK_EDGE_TRANSITION |
                        _SPI_MSB_FIRST |
                        _SPI_SS_DISABLE |
                        _SPI_SSM_ENABLE |
                        _SPI_SSI_1,
                        &_GPIO_MODULE_SPI1_PA56_PB5 );
}

#endif

// ============================================================================
//                                                                      FLASH
// ============================================================================

#if ( _MMB_MODULE_FLASH == 1 )

static sbit BSP_FLASH_CE at                 GPIOB_ODR.B11;

const static hal_spi_obj_t BSP_FLASH_SPI_MODULE =
{
    SPI2_Write,
    SPI2_Read
};

static void hal_flash_init ( void )
{
    //  Initialize BSP_FLASH_CE as output pin.

    GPIO_Digital_Output( &GPIOB_BASE, _GPIO_PINMASK_11 );

    //  Initialize SPI module.

    SPI2_Init_Advanced(
                        _SPI_FPCLK_DIV16,
                        _SPI_MASTER |
                        _SPI_8_BIT |
                        _SPI_CLK_IDLE_LOW |
                        _SPI_FIRST_CLK_EDGE_TRANSITION |
                        _SPI_MSB_FIRST |
                        _SPI_SS_DISABLE |
                        _SPI_SSM_ENABLE |
                        _SPI_SSI_1,
                        &_GPIO_MODULE_SPI2_PB13_14_15
                      );

    BSP_FLASH_CE = 1;
}

#endif

// ============================================================================
//                                                                        RGB
// ============================================================================

#if ( _MMB_MODULE_RGBLED == 1 )
#   ifdef _MIKROMEDIA_7_
#       ifdef __STM32__

//  Navigate to gpio.c file and update sbit variables related to Soft I2C.

static void hal_rgb_init ( void )
{
    //  Initialize I2C bitband.
    
    ---

    Delay_ms( 200 );
}

#       else

const static hal_i2c_obj_t BSP_RGB_I2C_MODULE =
{
    ---,
    ---,
    ---
};

static void hal_rgb_init ( void )
{
    //  Initialize I2C module.
    
    ---
}

#   endif
#else

static sbit BSP_RGB_RED        at                  ---;
static sbit BSP_RGB_GREEN      at                  ---;
static sbit BSP_RGB_BLUE       at                  ---;

static void hal_rgb_init ( void )
{
    //  Initialize all GPIO pins for R/G/B as outputs.
    
    ---
}

#endif
#endif

// ============================================================================
//                                                                         IR
// ============================================================================

#if ( _MMB_MODULE_IR == 1 )

static sbit BSP_IR_RX at                            ---;

static void hal_ir_init ( void )
{
    //  Initialize BSP_IR_TX as input pin.

    ---
}

#endif

// ============================================================================
//                                                                         RF
// ============================================================================

#if ( _MMB_MODULE_RF == 1 )

static sbit BSP_RF_CS at                            GPIOG_ODR.B9;
static sbit BSP_RF_CE at                            GPIOG_ODR.B12;
static sbit BSP_RF_INT at                           GPIOG_IDR.B10;

const static hal_spi_obj_t BSP_RF_SPI_MODULE =
{
    SPI2_Write,
    SPI2_Read
};

static void hal_rf_init ( void )
{
    //  Initialize BSP_RF_INT as input pin.

    GPIO_Digital_Input( &GPIOG_BASE, _GPIO_PINMASK_10 );

    //  Initialize BSP_RF_CE as output pin.

    GPIO_Digital_Output( &GPIOG_BASE, _GPIO_PINMASK_12 );

    //  Initialize BSP_RF_CS as output pin.

    GPIO_Digital_Output( &GPIOG_BASE, _GPIO_PINMASK_9);

    //  Initialize SPI module.

    SPI2_Init_Advanced(
                        _SPI_FPCLK_DIV16,
                        _SPI_MASTER |
                        _SPI_8_BIT |
                        _SPI_CLK_IDLE_LOW |
                        _SPI_FIRST_CLK_EDGE_TRANSITION |
                        _SPI_MSB_FIRST |
                        _SPI_SS_DISABLE |
                        _SPI_SSM_ENABLE |
                        _SPI_SSI_1,
                        &_GPIO_MODULE_SPI2_PB13_14_15
                      );

}

#endif

// ============================================================================
//                                                                        MP3
// ============================================================================

#if ( _MMB_MODULE_MP3 == 1 )


static volatile sbit BSP_MP3_DREQ at                 GPIOD_IDR.B9;
static volatile sbit BSP_MP3_CS at                   GPIOD_ODR.B11;
static volatile sbit BSP_MP3_RST at                  GPIOD_ODR.B8;
static volatile sbit BSP_MP3_DCS at                  GPIOD_ODR.B10;

static const hal_spi_obj_t BSP_MP3_SPI_MODULE =
{
    SPI2_Write,
    SPI2_Read
};

static void hal_mp3_init ( void )
{
    //  Initialize BSP_MP3_DREQ as input pin.

    GPIO_Digital_Input( &GPIOD_BASE, _GPIO_PINMASK_9 );

    //  Initialize BSP_MP3_CS as output pin.

    GPIO_Digital_Output( &GPIOD_BASE, _GPIO_PINMASK_11 );

    //  Initialize BSP_MP3_RST as output pin.

    GPIO_Digital_Output( &GPIOD_BASE, _GPIO_PINMASK_8 );

    //  Initialize BSP_MP3_DCS as output pin.

    GPIO_Digital_Output( &GPIOD_BASE, _GPIO_PINMASK_10 );

    //  Initialize SPI module.

    SPI2_Init_Advanced(
                        _SPI_FPCLK_DIV256,
                        _SPI_MASTER |
                        _SPI_8_BIT |
                        _SPI_CLK_IDLE_LOW |
                        _SPI_FIRST_CLK_EDGE_TRANSITION |
                        _SPI_MSB_FIRST |
                        _SPI_SS_DISABLE |
                        _SPI_SSM_ENABLE |
                        _SPI_SSI_1,
                        &_GPIO_MODULE_SPI2_PB13_14_15
                      );
}

#endif

// ============================================================================
//                                                                      ACCEL
// ============================================================================

#if ( _MMB_MODULE_ACCEL == 1 )

static sbit BSP_ACCEL_INT_1 at                          GPIOB_IDR.B9;
static sbit BSP_ACCEL_INT_2 at                          GPIOF_IDR.B2;

const static hal_i2c_obj_t BSP_ACCEL_I2C_MODULE =
{
    I2C1_Start,
    I2C1_Write,
    I2C1_Read
};

static void hal_accel_init ( void )
{
    //  Initialize BSP_ACCEL_INT_1 as input pin.

    GPIO_Digital_Input( &GPIOB_BASE, _GPIO_PINMASK_9 );

    //  Initialize BSP_ACCEL_INT_2 as input pin.

    GPIO_Digital_Input( &GPIOF_BASE, _GPIO_PINMASK_2 );

    //  Initialize I2C module.

    I2C1_Init_Advanced( 100000, &_GPIO_MODULE_I2C1_PB67 );
}

#endif

// ============================================================================
//                                                                        ALS
// ============================================================================

#if ( _MMB_MODULE_ALS == 1 )

const static hal_i2c_obj_t BSP_ALS_I2C_MODULE =
{
    ---, 
    ---,
    ---
};

static void hal_als_init ( void )
{
    //  Initialize I2C module.    

    ---
}

#endif

// ============================================================================
//                                                                       TEMP
// ============================================================================

#if ( _MMB_MODULE_TEMP == 1 )

#if ( JP_VREF == 0 )
#   define BSP_TEMP_ADC_VREF                ( 2.048f )
#elif ( JP_VREF == 1 )
#   define BSP_TEMP_ADC_VREF                ( 3.300f )
#else
#   error "REF SEL POSITION UNKNOWN"
#endif

#define BSP_TEMP_ADC_CHANNEL                ( --- )
#define BSP_TEMP_ADC_RESOLUTION             ( --- )

const static hal_adc_obj_t BSP_TEMP_ADC_MODULE =
{
    ---,
    ---
};

static void hal_temp_init ( void )
{
    // Set analog input to the TEMP Pin

    ---
    
    //  Initialize ADC module.

    ---
}

#endif

// ============================================================================
//                                                                       WIFI
// ============================================================================

#if ( _MMB_MODULE_WIFI == 1 )

static volatile sbit CC3100_HIB_PIN     at          GPIOB_ODR.B10;
static volatile sbit CC3100_CS_PIN      at          GPIOC_ODR.B8;
static volatile sbit CC3100_RST_PIN     at          GPIOG_ODR.B1;

const static hal_spi_obj_t BSP_WIFI_SPI_MODULE =
{
    SPI1_Write,
    SPI1_Read
};

static void hal_wifi_init ( void )
{
    //  Disable interrupt for WiFi module.

    NVIC_IntDisable( IVT_INT_EXTI0 );

    //  Initialize CC3100 HIB as input pin.

    GPIO_Digital_Output( &GPIOB_BASE, _GPIO_PINMASK_10 );

    //  Initialize CC3100 RST as input pin.

    GPIO_Digital_Output( &GPIOG_BASE, _GPIO_PINMASK_1 );

    //  Initialize CC3100 CS as output pin.

    GPIO_Digital_Output( &GPIOC_BASE, _GPIO_PINMASK_8 );

    //  Initialize CC3100 INT as input pin.

    GPIO_Digital_Input( &GPIOG_BASE, _GPIO_PINMASK_0 );

    /*
        Initialize CC3100 SPI peripheral.

        NOTE :

            SPI must be configured to SPI mode 0 ( IDLE LOW / FIRST EDGE ).
    */

    SPI1_Init_Advanced(
                        _SPI_FPCLK_DIV16,
                        _SPI_MASTER |
                        _SPI_8_BIT |
                        _SPI_CLK_IDLE_LOW |
                        _SPI_FIRST_CLK_EDGE_TRANSITION |
                        _SPI_MSB_FIRST |
                        _SPI_SS_DISABLE |
                        _SPI_SSM_ENABLE |
                        _SPI_SSI_1,
                        &_GPIO_MODULE_SPI1_PA56_PB5
                       );

    CC3100_CS_PIN = 1;
}
#endif

// ============================================================================
//                                                                     BUZZER
// ============================================================================

#if ( _MMB_MODULE_BUZZER == 1 )

static void hal_buzzer_init ( void )
{
    //  Initialize BUZZER pin as output.

    GPIO_Digital_Output( &GPIOB_BASE, _GPIO_PINMASK_8 );

    //  Initialize Sound library.

    Sound_Init( &GPIOB_ODR, 8 );
}

#endif

// ============================================================================
//                                                                   ETHERNET
// ============================================================================

#if ( _MMB_MODULE_ETHERNET == 1 )

#include "__Ethernet_Intern.h"

static void hal_ethernet_init ( uint8_t * mac, uint8_t * ip )
{
    // Initialize ETH library.

    
}

#endif

// ============================================================================
//                                                                        RTC
// ============================================================================

#if ( _MMB_MODULE_RTC == 1 )

static void hal_rtc_init ( void )
{
    //  Initialize RTC module.

    //  Enable RTC clock.

    RCC_APB1ENR.PWREN = 1;

    if ( RCC_BDCR.RTCEN == 0 )
    {
        //  Allow access to RTC.

        PWR_CR.DBP = 1;

        //  Unlock write protection.

        RTC_WPR = 0xCA;
        RTC_WPR = 0x53;

        /*
            Make it possible to change clock source.

                RTCEN = 1, LSE, LSEON
        */

        RCC_BDCR = 0x00010000;
        RCC_BDCR = 0x00008101;
    }

    //  Wait for RTC APB registers synchronization.

    while ( RTC_ISR.RSF != 1 )
    {
        //  Nothing to do here.
    }

    //  Wait till LSE is ready.

    while ( RCC_BDCR.LSERDY != 1 )
    {
        //  Nothing to do here.
    }

    RTC_WPR = 0xFF;
}

#endif

// ============================================================================
//                                                                        USB
// ============================================================================

#if ( _MMB_MODULE_USB == 1 )

//  Enables power for USB device.

static sbit BSP_USB_PSW at                          GPIOD_ODR.B6;

static void hal_usb_init ( void )
{
    //  Enable alternate GPIO function if needed

    GPIO_Alternate_Function_Enable( &_GPIO_MODULE_OTG_FS_PA89_10_11_12 );

    //  Initialize PWS pin as output.

    GPIO_Digital_Output( &GPIOD_BASE, _GPIO_PINMASK_6 );

    // PSW to 0 to turn on power for device.

    BSP_USB_PSW = 1;
    Delay_ms( 100 );
    BSP_USB_PSW = 0;
}

#endif

// ============================================================================
//                                                                    SHUTTLE
// ============================================================================

#if ( _MMB_MODULE_SHUTTLE == 1 )



#endif
#endif

/* -------------------------------------------------------------------------- */
/*
    mmb_mcu_---.h

  Copyright (c) 2017, MikroElektonika - http://www.mikroe.com

  All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. All advertising materials mentioning features or use of this software
   must display the following acknowledgment:
   This product includes software developed by the MikroElektonika.

4. Neither the name of the MikroElektonika nor the
   names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY MIKROELEKTRONIKA ''AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL MIKROELEKTRONIKA BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

----------------------------------------------------------------------------- */